# 3DS Merchant Backend sample

This repository holds a java web application that demonstrate how a 3DS merchant backend would receive an authentication request from mSIGNIA uSDK - either iOS or Android - transform/enrich it and send to a 3DS server.

## Configuration

See `resources/application.yml` for the two configuration properties:

* `threeds-server-auth-url` - this should be set a full URL to a 3DS Server's authentication endpoint
* `three-ds-requestor-id` - this should be set of authorized 3DS Requestor ID 

## How to run

Open the project in IntellijIDEA as a gradle project and run the `main` method. The app, once started, is configured to start a server on 8080 port. The authentication endpoint is available at `/api/v1/auth` path - i.e. `http:localhost:8080/api/v1/auth`

## The expected request

The endpoint expects a JSON document to be POSTed to it. The JSON document comprised of three data elements:

* `sdkMaxTimeout` - Indicates maximum amount of time (in minutes) for all exchanges. Recommended value to set is "60"
* `deviceRenderOptions` - Defines the SDK UI types that the device supports for displaying specific challenge user interfaces within the uSDK. Should be set by the mobile app. See 3DS spec for more details on this data element
* `authRequestParameters` - when the mobile app calls `transaction.getAuthenticationRequestParameters` method on the uSDK, the result it receives should be literally placed in this (`authRequestParameters`) data element.

## Sample request

Here's a `curl` command that can be used to trigger the flow:

```
curl --location --request POST 'localhost:8080/api/v1/auth' \
--header 'Content-Type: application/json' \
--data-raw '{
	"sdkMaxTimeout": "60",
	"deviceRenderOptions": {
		"sdkInterface": "03",
		"sdkUiType": [ "01", "02", "03", "04", "05" ]
	},
	"authRequestParameters":
		{
			"deviceData": "eyJraWQiOiJkZTc2NTViZS0zZDZmLTQ4MDItOWQ2OC1jMzZhOWQyM2U3MWMiLCJlbmMiOiJBMTI4R0NNIiwiYWxnIjoiUlNBLU9BRVAtMjU2In0.qq1kocXM1WsISWZaBJyaxDovcY23UDZ9BsDie3g1Oac3O8QuXusL2LnA8C1u6RvtlHJ0IT3RWmJg2ZjcUF7s1K0GKgQrsmyEbLG33H6g7i-5UqPY1diNBrzzRx6Y5mkTGap16JJ-WtBVtEClMrceK9jqU86VUZvsWdabBolDwm6bQFTyKKoh5Yk0-q63QS9eVqZA5kVXn3CLsObF8Y-uqwOPaysCIA5j-o9-j6409qJk3Lc0F2Bl4ht17aDjtbdzVXut-fb5gWP7577vfG4deDZMMZWW2qFNEPh9FEpDi6V6IjomS9CBQHP2ceji42Zwb3FT4uId49eICk1J0LPZPw.154HO1Cel7LsbDQy.lxYHoaLs-3IhGVR6_TsW7tL0Nw5YNlGN-taMROPVh8_Ks3fRJ1RJqgDHUtO5T3jFJlrUYYlJ_g06202N3MNU4RKQG4T-9Vl1zdTv4QgQxIDhCKzawW9TDffFG957_Vl3okhol7nj5AvC3u_o4TOYx20jYbI29RTyFD5WM3QrexTQATDuyRNGpd-xAgZRlOaWbJPKEZvzg9OMSCO7U99s1DqHguKqQmdfMaRLAdvtpawpMP4lc8Ia7yqbrfqI41-ECjg0VL2Kjdj3W-LH4Zkmd2ygN-IjVb-YfFjOw64UwflpVjYmYuiPspt6SSe4ftVlxmushzgt_zG8q1NZWkW8UWENPYlFJVW7ERDyzlB7zJ23KWpKPx4TMumVIN83rlGuNUoP1Rq0osiK5igOWJt9dbtRT5L1SXPoHkej8IDfbGBqjUCvJDEwUrQX-7wkjd8QV32TKkF7u-csnuT4_89mbUXh_wAA4Y8JV4udCa9PsQRE76UGhPICDmvJpphtRa4oyXpCE1rDOg77YwadCOfNPgRxiuadwJPc-BzLuAQQAlJQ5J37MKNdrs9waJD3OzjGGoKCr4Qe-A4YcVLInjbp8gb-mvy8AIJhSSl93c91gObIuheXaxIt_BqteqE6rWspO3tV4i_KEZe2rTQE0pgLoDi45D5JoUKrxs08iyfFZ1brvbIQ_a9nL5uLj2A-XqWTAkz8jY0S3-KDecVFTMIAxVSsK8WolOOginny9wshGVZhiSIQbrrIuenNOAvc8LK7WtDHeB__2Bj50Yt1Ov1pn_vIaWw21apzTFS0joNJsY3QEWIOrpx28GsAujwYPxNrlttqG78qdVffYwO_iNn8yCU7Jox00UepDI9_9w9lz5JIfK1bf-5yFHccxT9LzjOmbFp5Q6HGfq6M-Jr8jQAfHIn4GP4BtoOXRjMV4mLprHeVzS0efVo2hmDjWka3VYM8bgxfVg31ezdTM0_tvxBXhgQTl04xAVYPLNKqwbPPqtaDE6AikHB1hckYqfBYdN2MgRI9YiYyiuSw3VuLJIFOy8tmlfaK4IevZHmN_hwW9MdNq18RhitfLqbTE8748X6ewS_d-_k4OrqRLOrZA9RJfUxwrjh66JJqi0Zpqrzj3dn1l9Z8_PMU17Ia_90hrRNZMqgpSbI_Oe3BUasMlmD9cL-q9MV18JpxExslczejpTYFc-G2C4zoZLhoktQCDd1ffzhwINE-rUxeiSljHj4hgUCfVNP86DKIt40puc9FZDgAgM2odr6KdUR0XZxkLm__DRzy1kpDoVG6CXNK3xgAywB2WDpEK8e1vT7KCpIzRQkGynV9ysX1nCUM2MxM4WH2UgJvotR4f3H0qvtiLbUA3hfXODI2HlVVVAhz5kjuL_7hhD18vs3ELZnMM0fXMlaRIpuj8bdYIlYZVBHMZpmsAnUjTvBpicDjRiKkphgaeIDMuS5K74f8ziNJuMBYT5EcHbpDInUHGkGXNnkkqdJR7UOBr3P8Lh5KRBCqO6spOj7Q5exsXn0_OF7bbVstxxW_w8a6SsfAFiDgdqr8ebZZ7hhNO3TFhVpwlh-QUzxrUUZ17XIrutWnQVFtTXHmSO5dO7JxLXzsJ-QjiB1xL3j8d4WDOMaBt8056i08q_d_c1eEQkcY67y3lfXWu3k5BrB0nOxRaJJQBkEfF97Hw1oTpmuqsPFlsq_34hbT4Aef-79AjCVTmhNcj4SBZwz8gcCX6wJJm1nqcfXtgd8SL7oToPdY6yeANuONpmJ4wKgCWxU8Adm0wNiLi5imitf4eYLCTDN46HGBb1e_ATrSisjmC7c5kl617vX1PDldS97fd5F2HaZ8IlBnLOYUvtjTcqTB1EWY4Cdfoo4cqjx8qw8Qq58kEs0DKX953hjRkgtwCSIv2lYVP_Q7UHSFWCDI_ltSskbqNG4Bbc_fYE7zGOtMQK7_4F2KggPa-hLDp4PikGEawisjiru78WcN4GiNgELhPCVJFhJxJQplTmM2gxjd7slNrii_rIp75Ub-gXN-1f0Gxn2gDYVLbUKSWpFi6WWuQ2Okt6d4zWQzCvb3J1cWysl16CJ8UubcXiorfvg0iSSP6AJXwOu2X5wOCLufh7beNJ10mWfTVaUCN0AolXm0K0lUGPiQBS44JIB5EweLccRxGLD_O8mWbstmWYwNYw31sXLRuPNrm2QH8NZBJuNyE5yTy1nXZYViNT0_kIYuUeLRxYk0Pyk4SjwzIJ8pVfBKjKKY_CkEPKU4UdJ5_MqW825O6dNAvPPfDcSJxp-qCJbq1GdaAXowADYlFqcuoMQBRUoXWeiR9hKbUu9ti9dJsfqQUBeKdCqS6ZmL_PkI3CRN8TOpPAY4Nu4GJCyFYKD4nBODml_Zy_IQGOHTvqkLPGHr_db3ZT7SgGHRkLkPtNtVy0KVOjCXWxEcPvPYbWqXdSYJZ3rd4EhUYvJvuSWEKqF0KcJEIlrkKNr7b4eGfP0Bvai59JlDbNzzCb7dhjP7EAJch5vJkTOITyHR9j1xd_cSiaA1PNcva5ogQR0LkgY3fpBhDVP7Drmx5W24RWL4-zU4VrdDLSZbC3dlLj1nRXI1grx2KOK6bradsJwb4Dki5HlH4AHBqrheJNjf4NtCh7vnnjkFZ6SBl_KBBg6s58UWldI9PFDXfxIf6aYv_sMgaqI6_VjFEVmT3ztI03Juv5zhmKRYsbXxsXzZZ8YIjKD-sxN3FnkT-xin0RhBKrfGCbHz_oSD4XJkzhc19oLE_rZblK9Vn7BW9iK0K_2mhMM0_sJ8H5YiS6OBFwgtKCTIDjyK-vqBE6o13tIQHYiEfAHLQ6Qp8XxquqBzOOKVc9AXuMMYxfOVXnzZ6V83TtMzbBmqUoM-r8ELvMreLScN-_dCa6rCz4S5P8wSOeCMV3g8O8NZKOnQQTLriZ3AOKLcUpjGUa1CEjL9XnyIRZpLpzNlFnXtgDU-B1iyDZ1WNh4AvwNWUo-YmiGIYKZn0ZMWLEv4-0s6XL1sat0DYKvIfGv5pmvl7lajlTQat657yNLCwuY3Y4MLZt0KCD0kej8qgFimQsYegRYit-UuoqsMj6w8uerKPiao8v4hG2TOcrdpNVb3mFAgVLivIH87hAVxzQRgE4slEInPgsGTKUIY0gSu2LWB5q_KAmynyqnPwfkScfE_42sXZVxnUtEz38P98ZfYl9i40OwPjXHOv5D9fW09n-3maLbeC0sxYfGaEWLkHxoheVzIgmgVTpQ1_eO9PLK4CAm0uqtiEbdbJQVHhrs1K9CGSO9gtXNg3-a32R-XgofYBmOJFLt1J50s2cTGrcrp8yCLeGZdNs2w6ogROcUgocJAcQwCURR7ELzQK3ckDpR-5TFHeDreMf39k1xNBJvhDmMo46QsLYZP1EVBqTmKvrqUNqqUlaq9zsI1quGdwW5FUhcEptFRA-48MnkDJQmgIIi_D53nB6Ztg5Sj-U2Xsw7Rm5KltBv05MFSe79VnAVBAny_OydUetOo4Uo9bx7NS6rZBaEP7UKTle3kjhHUpy1-PA7foinBC2i3hZ-6Zkpy3LY_k4gKTXzi7nN6FFdoP4BZYtZfFvOJeUFPk2hkZ4heP_GMQLM69WwWKf6zOMDzkMkIY_Szs6Ag9ZDdk--jLLsXUYHUc8t6TSDRB3g_PSVSye6MC5WTSnoHZF2TZPmTZHMN8xT0gHbXMbqRRl8cRVTZTsEkXjjkHTV3FzggQ5KVM0HTqohX72rJiF--QTV5UbxWifl5Mv3s6DX6NiMJ9BxjrW0Xfcfn7jw-ZplNZxAEbvoJpvXGTILzb6SZiNaBwytzDW7p0JUCzWevvrJDqhAJYKljXMTe4pEQwKwHIuSTxoudvl6Ny5Uwe7diSc65j-ZVRKNFC5FXWEccTPRn8av4xsEpwE092kvtbtO3uvoT5QmMJQqTkRYSj27ikNVzD4x3NaAmLSN_muCEDdw2cu0IovpZbmOY0smA3xQBU6kjYg.eEF1yh7uJ2ohWyahtK02GA",
			"sdkTransactionID": "3c254a9a-0bd7-4925-8aed-8a479ab4f703",
			"sdkAppID": "0f1887f1-7a2d-40c1-9865-988ce68dc807",
			"sdkReferenceNumber": "3DS_LOA_SDK_MSIG_020100_00111",
			"sdkEphemeralPublicKey": "{'\''kty'\'':'\''EC'\'','\''crv'\'':'\''P-256'\'','\''x'\'':'\''SVqB4JcUD6lsfvqMr-OKUNUphdNn64Eay60978ZlL74'\'','\''y'\'':'\''lf0u0pMj4lGAzZix5u4Cm5CMQIgMNpkwy163wtKYVKI'\'','\''d'\'':'\''0g5vAEKzugrXaRbgKG0Tj2qJ5lMP4Bezds1_sTybkfk'\''}",
			"messageVersion": "2.1.0"
		}
}'
```