/*
 Copyright © mSIGNIA, Incorporated, 2007.  All rights reserved.

 Software is protected by one or more of U.S. Patent No. 9,559,852, 9294448, 8,817,984,
 international patents and others pending. For more information see www.mSIGNIA.com.  User agrees
 that they will not them self, or through any affiliate, agent or other third-party, entity or
 other business structure remove, alter, cover or obfuscate any copyright notices or other
 proprietary rights notices of mSIGNIA or its licensors.  User agrees that they will not them
 self, or through any affiliate, agent or other third party, entity or other business structure
 (a) reproduce, sell, lease, license or sublicense this software or any part thereof, (b)
 decompile, disassemble, re-program, reverse engineer or otherwise attempt to derive or modify
 this software in whole or in part, (c) write or develop any derivative software or any other
 software program based upon this software, (d) provide, copy, transmit, disclose, divulge, or
 make available to, or permit use of this software by any third party or entity or machine without
 software owner's prior written consent, (e) circumvent or disable any security or other
 technological features or measures used by this software.
*/

package com.msignia.samples.threeds.backend.components;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;

public class BrowserIPAddress {
    private final Logger logger = LoggerFactory.getLogger(BrowserIPAddress.class);

    private final HttpServletRequest httpServletRequest;

    public BrowserIPAddress(HttpServletRequest httpServletRequest) {
        this.httpServletRequest = httpServletRequest;
    }

    public String getIP() {

        List<String> ipHeaders = Arrays.asList("x-real-ip", "x-original-forwarded-for", "x-forwarded-for");

        String ipAddress = null;

        for (String headerName : ipHeaders) {
            if (!StringUtils.isEmpty(httpServletRequest.getHeader(headerName))) {
                ipAddress = httpServletRequest.getHeader(headerName).split(",")[0].strip();
                logger.debug("IP Address {} has been detected from the header {}", ipAddress, headerName);
                break;
            }
        }

        if (StringUtils.isEmpty(ipAddress)) {
            ipAddress = httpServletRequest.getRemoteAddr();
            logger.debug("Return IP Address {} from the request", ipAddress);
        }

        return  ipAddress;
    }
}
