package com.msignia.samples.threeds.backend.services;

import com.msignia.samples.threeds.backend.logger.TransactionLogger;
import com.msignia.threeds.mpi.get_supported_versions.GetSupportedVersionsParameters;
import com.msignia.threeds.mpi.get_supported_versions.GetSupportedVersionsResponse;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

@Service
public class SupportedVersionsService {

    private final Logger logger = LoggerFactory.getLogger(AuthenticationServiceImpl.class);

    private final ThreeDsServerClientByMPI threeDsServerClient;
    private final TransactionLogger transactionLogger;
    private final String threeDSRequestorID;

    public SupportedVersionsService(ThreeDsServerClientByMPI threeDsServerClient,
                                    TransactionLogger transactionLogger,
                                    @Value("${app.three-ds-requestor-id}") String threeDSRequestorID) {
        this.threeDsServerClient = threeDsServerClient;
        this.transactionLogger = transactionLogger;
        this.threeDSRequestorID = threeDSRequestorID;
    }

    // The user BIN has to be read from the database here and sent to the 3DS server for getting a list
    // of supported versions for it. For the sake of the sample app simplicity however we hardcode a card number
    // as if it has been read from the database...
    public GetSupportedVersionsResponse perform(String email, JSONObject requestParams) throws Exception {
        logger.debug("perform entered...");

        logger.debug("sending request tp supported-versions: {}", requestParams);
        GetSupportedVersionsParameters supportedVersionsParameters = new GetSupportedVersionsParameters();

        supportedVersionsParameters.setAcctNumber(StringUtils.isEmpty(requestParams.optString("acctNumber")) ?
                requestParams.getString("cardID") :
                requestParams.getString("acctNumber"));
        supportedVersionsParameters.setThreeDSRequestorID(threeDSRequestorID);
        supportedVersionsParameters.setEmail(email);

        GetSupportedVersionsResponse result = threeDsServerClient.getSupportedVersions(supportedVersionsParameters);

        String threeDSServerTransID = result.get(0).getThreeDSServerTransID();

        TransactionLogger.LogEntry entryRequest = new TransactionLogger.LogEntry.Builder()
                .url(null)
                .payload(requestParams.toString())
                .headers(null)
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.SUPPORTED_VERSIONS_V1, "01", entryRequest);

        TransactionLogger.LogEntry entryResponse = new TransactionLogger.LogEntry.Builder()
                .url(null)
                .payload(new JSONArray(result).toString())
                .headers(null)
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.SUPPORTED_VERSIONS_V1, "04", entryResponse);



        return result;
    }
}
