package com.msignia.samples.threeds.backend.services;

import com.msignia.samples.threeds.backend.logger.TransactionLogger;
import okhttp3.*;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Service
public class ThreeDsServerClient {
    private final Logger logger = LoggerFactory.getLogger(AuthenticationServiceImpl.class);

    private final OkHttpClient okHttpClient;
    private final String threeDsServerAuthUrl;
    private final String threeDsServerSupportedVersionsUrl;
    private final TransactionLogger transactionLogger;

    public ThreeDsServerClient(
            OkHttpClient okHttpClient,
            @Value("${app.threeds-server-auth-url}") String threeDsServerAuthUrl,
            @Value("${app.threeds-server-supported-versions-url}") String threeDsServerSupportedVersionsUrl,
            TransactionLogger transactionLogger) {

        this.okHttpClient = okHttpClient;
        this.threeDsServerAuthUrl = threeDsServerAuthUrl;
        this.threeDsServerSupportedVersionsUrl = threeDsServerSupportedVersionsUrl;
        this.transactionLogger = transactionLogger;
    }

    public JSONObject authenticate(JSONObject payload) throws Exception {

        JSONObject response = post(payload, threeDsServerAuthUrl);

        String threeDSServerTransID = response.getString("threeDSServerTransID");

        TransactionLogger.LogEntry entryRequest = new TransactionLogger.LogEntry.Builder()
                .url(threeDsServerAuthUrl)
                .headers(null)
                .payload(payload.toString())
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.AREQ_ARES_V1,
                new TransactionLogger.LogPath.Builder()
                        .from(TransactionLogger.Requestor.MERCHANT_BACKEND)
                        .to(TransactionLogger.Requestor.THREE_DS_SERVER)
                        .build().getRoute(),
                entryRequest);

        TransactionLogger.LogEntry entryResponse = new TransactionLogger.LogEntry.Builder()
                .url(threeDsServerAuthUrl)
                .headers(null)
                .payload(response.toString())
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.AREQ_ARES_V1,
                new TransactionLogger.LogPath.Builder()
                        .from(TransactionLogger.Requestor.THREE_DS_SERVER)
                        .to(TransactionLogger.Requestor.MERCHANT_BACKEND)
                        .build().getRoute(),
                entryResponse);

        return response;
    }

    public JSONObject getSupportedVersions(JSONObject payload) throws Exception {
        JSONObject response = post(payload, threeDsServerSupportedVersionsUrl);

        String threeDSServerTransID = response.getString("threeDSServerTransID");

        TransactionLogger.LogEntry entryRequest = new TransactionLogger.LogEntry.Builder()
                .url(threeDsServerSupportedVersionsUrl)
                .payload(payload.toString())
                .headers(null)
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.SUPPORTED_VERSIONS_V1, "02", entryRequest);

        TransactionLogger.LogEntry entryResponse = new TransactionLogger.LogEntry.Builder()
                .url(threeDsServerSupportedVersionsUrl)
                .payload(response.toString())
                .headers(null)
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.SUPPORTED_VERSIONS_V1, "03", entryResponse);


        return response;
    }

    private JSONObject post(JSONObject payload, String url) throws Exception {
        RequestBody body = RequestBody.create(payload.toString(), MediaType.parse("application/json"));
        logger.debug("Sending request: {}/{}", url, payload.toString(4));

        Request.Builder requestBuilder = new Request.Builder().url(url).post(body);

        Request request = requestBuilder.build();

        try (Response response = okHttpClient.newCall(request).execute()) {
            return new JSONObject(response.body().string());
        }
    }
}
