package com.msignia.samples.threeds.backend.services;

import com.msignia.samples.threeds.backend.mpi.config.ThreedsServerConfig;
import com.msignia.samples.threeds.backend.logger.TransactionLogger;
import com.msignia.threeds.mpi.ThreeDsMerchantPlugin;
import com.msignia.threeds.mpi.authentication.AuthenticationParameters;
import com.msignia.threeds.mpi.authentication.AuthenticationResponse;
import com.msignia.threeds.mpi.get_supported_versions.GetSupportedVersionsParameters;
import com.msignia.threeds.mpi.get_supported_versions.GetSupportedVersionsResponse;
import com.msignia.threeds.mpi.impl.PluginImpl;
import com.msignia.threeds.mpi.impl.model.TlsParameters;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

@Service
public class ThreeDsServerClientByMPI {
    private final Logger logger = LoggerFactory.getLogger(AuthenticationServiceImpl.class);

    private final TransactionLogger transactionLogger;
    private final ThreedsServerConfig threedsServerConfig;

    public ThreeDsServerClientByMPI(TransactionLogger transactionLogger,
                                    ThreedsServerConfig threedsServerConfig) {
        this.transactionLogger = transactionLogger;
        this.threedsServerConfig = threedsServerConfig;
    }

    public AuthenticationResponse authenticate(AuthenticationParameters authParams) throws Exception {
        PluginImpl plugin = (PluginImpl) createMpiPlugin();
        AuthenticationResponse response = plugin.authenticate(authParams);

        String threeDSServerTransID = response.getThreeDSServerTransID();

        TransactionLogger.LogEntry entryRequest = new TransactionLogger.LogEntry.Builder()
                .url(threedsServerConfig.getThreeDsServerAuthEndpoint())
                .headers(null)
                .payload(new JSONObject(authParams).toString())
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.AREQ_ARES_V1,
                new TransactionLogger.LogPath.Builder()
                        .from(TransactionLogger.Requestor.MERCHANT_BACKEND)
                        .to(TransactionLogger.Requestor.THREE_DS_SERVER)
                        .build().getRoute(),
                entryRequest);

        TransactionLogger.LogEntry entryResponse = new TransactionLogger.LogEntry.Builder()
                .url(threedsServerConfig.getThreeDsServerAuthEndpoint())
                .headers(null)
                .payload(new JSONObject(response).toString())
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.AREQ_ARES_V1,
                new TransactionLogger.LogPath.Builder()
                        .from(TransactionLogger.Requestor.THREE_DS_SERVER)
                        .to(TransactionLogger.Requestor.MERCHANT_BACKEND)
                        .build().getRoute(),
                entryResponse);

        return response;
    }

    public GetSupportedVersionsResponse getSupportedVersions(GetSupportedVersionsParameters params) throws Exception {
        PluginImpl plugin = (PluginImpl) createMpiPlugin();
        GetSupportedVersionsResponse getSupportedVersionsResponse = plugin.getSupportedVersions(params);

        String threeDSServerTransID = getSupportedVersionsResponse.get(0).getThreeDSServerTransID();

        TransactionLogger.LogEntry entryRequest = new TransactionLogger.LogEntry.Builder()
                .url(threedsServerConfig.getThreeDsServerSupportedVersionsEndpoint())
                .payload(new JSONObject(params).toString())
                .headers(null)
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.SUPPORTED_VERSIONS_V1, "02", entryRequest);

        TransactionLogger.LogEntry entryResponse = new TransactionLogger.LogEntry.Builder()
                .url(threedsServerConfig.getThreeDsServerSupportedVersionsEndpoint())
                .payload(new JSONArray(getSupportedVersionsResponse).toString())
                .headers(null)
                .build();
        transactionLogger.log(threeDSServerTransID, TransactionLogger.Flow.SUPPORTED_VERSIONS_V1, "03", entryResponse);


        return getSupportedVersionsResponse;
    }

    protected ThreeDsMerchantPlugin createMpiPlugin() {
        PluginImpl.Builder builder = new PluginImpl.Builder()
                .threeDsServerUrl(threedsServerConfig.getThreeDsServerUrl())
                .authenticationEndpoint(threedsServerConfig.getThreeDsServerAuthEndpoint())
                .supportedVersionsEndpoint(threedsServerConfig.getThreeDsServerSupportedVersionsEndpoint())
                .version(2)
                .tlsParameters(new TlsParameters())
                .testPlatform();

        return builder.build();
    }
}
